package com.outr.webcommunicator

import netty.communicator.NettyCommunicatorManager
import java.util.UUID
import netty._
import handler.{PathHandler, RequestHandler, UploadHandler}
import org.jboss.netty.channel.{MessageEvent, ChannelHandlerContext}
import org.jboss.netty.handler.codec.http.HttpRequest
import org.jboss.netty.handler.codec.http.multipart.{Attribute, FileUpload}

/**
 * @author Matt Hicks <mhicks@powerscala.org>
 */
object TestWebapp extends NettyCommunicatorManager[TestCommunicator] {
  override def webSocketEnabled = false

  protected def instantiate(id: UUID) = new TestCommunicator(id)

  def main(args: Array[String]): Unit = {
    bind("localhost", 7000)
    register(PathHandler("/", ""))
    register(new WebResource {
      def request(webapp: NettyWebapp, context: ChannelHandlerContext, event: MessageEvent) = event.getMessage match {
        case request: HttpRequest => if (request.path == "/uploader") {
          Some(new UploadHandler() {
            def receivedAttribute(attribute: Attribute) = println("Attribute recieved: %s -> %s".format(attribute.getName, attribute.getValue))

            def receivedFileUpload(upload: FileUpload) = println("File received: %s".format(upload.getFilename))

            def sendResponse(context: ChannelHandlerContext) = RequestHandler.sendResponse(context)
          })
        } else {
          None
        }
      }
    })
    println("Server started...")
  }
}

class TestCommunicator(val id: UUID) extends Communicator {
  def receive(event: String, id: String, message: String) = event match {
    case "pre" => println("%s - Received: %s (%s)".format(id, event, message))
    case "post" => {
      println("Received post...sending testResponse!")
      send("testResponse", "Wahoo!")
    }
    case "chat" => send("chatResponse", message.reverse)
  }

  override def dispose() {    // TODO: verify the connection is disposing properly always
    super.dispose()

    println("%s - Argh! I'm dying!!!".format(id))
  }
}
