package com.outr.webcommunicator.netty

import handler.RequestHandler
import org.jboss.netty.channel.{MessageEvent, ChannelHandlerContext}
import org.jboss.netty.handler.codec.http.HttpRequest
import com.outr.webcommunicator.URL

/**
 * @author Matt Hicks <matt@outr.com>
 */
trait URLWebResource extends WebResource {
  def isMatch(url: URL): Boolean

  def create(request: HttpRequest): RequestHandler

  def request(webapp: NettyWebapp, context: ChannelHandlerContext, event: MessageEvent) = event.getMessage match {
    case request: HttpRequest if (isMatch(request)) => Some(create(request))
    case _ => None
  }
}

object URLWebResource {
  def apply(f: => RequestHandler)(matcher: URL => Boolean) = new URLWebResource {
    def isMatch(url: URL) = matcher(url)

    def create(request: HttpRequest) = f
  }

  def apply(regex: String, f: => RequestHandler) = new URLWebResource {
    def isMatch(url: URL) = url.toString.matches(regex)

    def create(request: HttpRequest) = f
  }
}