package com.outr.webcommunicator.netty.communicator

import com.outr.webcommunicator.netty.handler.AJAXHandler
import org.jboss.netty.channel.ChannelHandlerContext
import org.jboss.netty.handler.codec.http.HttpRequest
import org.jboss.netty.util.CharsetUtil

import com.outr.webcommunicator.{IdentifiableMessage, Communicator, CommunicatorReceiver, Event}
import org.powerscala.concurrent.Executor

import org.powerscala.concurrent.Time._

import org.powerscala.json._

/**
 * @author Matt Hicks <matt@outr.com>
 */
class CommunicatorAJAXPollingHandler[C <: Communicator]
        (val manager: NettyCommunicatorManager[C], timeout: Double = 15.seconds)
            extends AJAXHandler with CommunicatorReceiver[C] {
  def handle(context: ChannelHandlerContext, request: HttpRequest) = {
    val content = request.getContent
    val data = content.toString(CharsetUtil.UTF_8)
    val event = parse[IdentifiableMessage](data)
    if (handle(event)) {
      checkQueue()
      if (communicator != null) {
        communicator.updated()

        Executor.schedule(timeout) {
          if (communicator != null && communicator.communicatorReceiver == CommunicatorAJAXPollingHandler.this && context.getChannel.isOpen) {
            sendNoop()
          }
        }
      }
    } else {
      sendReload()
    }
  }

  override def outgoingEnqueued() = {
    super.outgoingEnqueued()

    checkQueue()
  }

  def checkQueue() = {
    if (communicator != null) {
      val queue = communicator.outgoing(lastSentId)
      if (queue.nonEmpty) {
        sendQueue(queue)
      }
    }
  }

  override def communicatorUpdating() {
    super.communicatorUpdating()

    if (context != null && context.getChannel.isOpen) {
      communicator.updated()    // Always keep this updated so it never times out
    } else {
      if (communicator.communicatorReceiver == this) {
        communicator.communicatorReceiver = null
      }
      communicator = null.asInstanceOf[C]
    }
  }

  def sendQueue(messages: Seq[IdentifiableMessage]) = {
    communicator.communicatorReceiver = null
    val data = generate(messages.toArray)
    streamString(data, context, request, "application/json", enableCaching = false)
    communicator.updated()
  }

  def sendNoop() = {
    communicator.communicatorReceiver = null
    val data = generate(Event("noop", null, null))
    streamString(data, context, request, "application/json", enableCaching = false)
    communicator.updated()
  }

  def sendReload() = {
    if (communicator != null) {
      communicator.communicatorReceiver = null
    }
    val data = generate(IdentifiableMessage(-1, "reload", "-1", null))
    streamString(data, context, request, "application/json", enableCaching = false)
    if (communicator != null) {
      communicator.updated()
    }
  }
}
