package com.outr.webcommunicator.netty.communicator

import com.outr.webcommunicator._
import com.outr.webcommunicator.netty.handler.AJAXHandler
import org.jboss.netty.channel.ChannelHandlerContext
import org.jboss.netty.handler.codec.http.HttpRequest
import org.jboss.netty.util.CharsetUtil
import java.util.UUID

import org.powerscala.json._

/**
 * @author Matt Hicks <matt@outr.com>
 */
class CommunicatorAJAXReceivingHandler[C <: Communicator]
        (val manager: NettyCommunicatorManager[C])
          extends AJAXHandler with CommunicatorSender[C] {
  def handle(context: ChannelHandlerContext, request: HttpRequest) = {
    val content = request.getContent
    val data = content.toString(CharsetUtil.UTF_8)
    val parsed = parse[SendEvent](data)
    val event = parsed.copy(messages = parsed.messages.asInstanceOf[List[Any]].map(v => JSONConverter.parseJSON[IdentifiableMessage](v)))
    val id = UUID.fromString(event.uuid)
    manager(id) match {
      case Some(c) => {
        communicator = c
        communicator.updated()
        communicator.communicatorSender = this
        communicatorChanged()
        event.messages.foreach(communicator.incoming)
      }
      case None => println("*** CommunicatorSender.handle - Unsupported reconnect when Communicator not found by id")
    }
    streamString("", context, request, "text/plain", enableCaching = false)
  }

  override def communicatorUpdating() {
    super.communicatorUpdating()

    if (context != null && context.getChannel.isOpen) {
      communicator.updated()    // Always keep this updated so it never times out
    } else {
      if (communicator.communicatorSender == this) {
        communicator.communicatorSender = null
      }
      communicator = null.asInstanceOf[C]
    }
  }
}