package com.outr.webcommunicator.netty.handler

import com.outr.webcommunicator.netty._
import com.outr.webcommunicator.URL
import org.jboss.netty.channel.{MessageEvent, ChannelHandlerContext}
import org.jboss.netty.handler.codec.http.HttpRequest
import java.io.File

/**
 * @author Matt Hicks <matt@outr.com>
 */
trait PathHandler extends WebResource with RequestHandler {
  def caching = true

  def lookup(url: URL): java.net.URL

  def request(webapp: NettyWebapp, context: ChannelHandlerContext, event: MessageEvent) = event.getMessage match {
    case request: HttpRequest if (lookup(request) != null) => Some(this)
    case _ => None
  }

  def apply(webapp: NettyWebapp, context: ChannelHandlerContext, event: MessageEvent) = event.getMessage match {
    case request: HttpRequest => streamURL(lookup(request), context, request)
  }
}

object PathHandler {
  val ClassLoading = (path: String) => getClass.getClassLoader.getResource(path)
  val FileLoading = (path: String) => {
    val file = new File(path)
    if (file.exists()) {
      file.toURI.toURL
    } else {
      null
    }
  }

  def apply(publicBasePath: String, internalBasePath: String, f: String => java.net.URL = ClassLoading, cacheContent: Boolean = true) = new PathHandler {
    override def caching = cacheContent

    def lookup(url: URL) = if (url.path.startsWith(publicBasePath) && url.path.length > publicBasePath.length) {
      val internalPath = "%s%s".format(internalBasePath, url.path.substring(publicBasePath.length))
      f(internalPath)
    } else {
      null
    }
  }
}