package com.outr.webcommunicator.netty.handler

import com.outr.webcommunicator.netty._
import org.jboss.netty.channel.{MessageEvent, ChannelHandlerContext}
import org.jboss.netty.handler.codec.http.{HttpHeaders, HttpRequest}
import org.jboss.netty.handler.codec.http.websocketx._

/**
 * @author Matt Hicks <mhicks@powerscala.org>
 */
abstract class WebSocketHandler extends RequestHandler {
  protected var handshaker: WebSocketServerHandshaker = null
  protected var context: ChannelHandlerContext = null

  def apply(webapp: NettyWebapp, context: ChannelHandlerContext, event: MessageEvent) = event.getMessage match {
    case request: HttpRequest => handshake(context, request)
    case frame: WebSocketFrame => handle(context, frame)
  }

  protected def handshake(context: ChannelHandlerContext, request: HttpRequest) = {
    val subprotocols: String = null
    val allowExtensions = false
    val wsFactory = new WebSocketServerHandshakerFactory(webSocketURL(request), subprotocols, allowExtensions)
    handshaker = wsFactory.newHandshaker(request)
    if (handshaker == null) {
      wsFactory.sendUnsupportedWebSocketVersionResponse(context.getChannel)
    } else {
      handshaker.handshake(context.getChannel, request).addListener(WebSocketServerHandshaker.HANDSHAKE_LISTENER)
      this.context = context
    }
  }

  protected def handle(context: ChannelHandlerContext, frame: WebSocketFrame) = frame match {
    case f: CloseWebSocketFrame => {
      handshaker.close(context.getChannel, f)
      handshaker = null
      this.context = null
    }
    case f: PingWebSocketFrame => context.getChannel.write(new PongWebSocketFrame(frame.getBinaryData))
    case f: PongWebSocketFrame => debug("PongWebSocketFrame")
    case f: TextWebSocketFrame => handleText(context, f)
    case f: BinaryWebSocketFrame => handleBinary(context, f)
    case _ => throw new UnsupportedOperationException("%s frame types not supported".format(frame.getClass))
  }

  def webSocketURL(request: HttpRequest) = "ws://%s%s".format(request.getHeader(HttpHeaders.Names.HOST), request.path)

  def handleText(context: ChannelHandlerContext, frame: TextWebSocketFrame): Unit

  def handleBinary(context: ChannelHandlerContext, frame: BinaryWebSocketFrame): Unit

  def send(data: String) = if (context != null) {
    context.getChannel.write(new TextWebSocketFrame(data))
  }
}
